﻿/*

 * 名称：Class_CmdMessgage
 * 
 * 程序员：王维攀
 * 
 * 创建日期：2016.03.28
 * 
 * 最后一次修改日期：2016.03.29
 * 
 * 备注：该文件主要定义了帧格式协议当中的几个命令字和数据，其他的命令请参看《ZEL6000系列帧格式通信协议》中的详细说明；该文件还包括帧格式控制报文的组装程序。
 
 
 */

using System;
using System.Collections.Generic;

public class Class_CmdMessgage
{
    public static byte CMD_AckCheckout = 0x12;
    public static byte CMD_SetControlMode = 0x20;
    public static byte CMD_SetWorkMode = 0x28;
    public static byte CMD_SetONOFF = 0x21;
    public static byte CMD_SetModeCCValue = 0x2A;
    public static byte CMD_GetCurrentValueNow = 0x63;
    public static byte CMD_GetVoltageValueNow = 0x65;
    public static byte CMD_GetAllValueAndStatusNow = 0x5F;

    public static byte ControlModeSel_Local = 0x00;
    public static byte ControlModeSel_PC = 0x01;

    public static byte CMD_SetON = 0x01;
    public static byte CMD_SetOFF = 0x00;

    public static byte CMD_SetWorkMode_CC = 0x00;
    public static byte CMD_SetWorkMode_CV = 0x01;
    public static byte CMD_SetWorkMode_CW = 0x02;
    public static byte CMD_SetWorkMode_CR = 0x03;

    public Class_CmdMessgage()
	{
	}

    public static byte[] CreateOneByteDataCmdMessage(byte SlaveAddr, byte Cmd, byte Data)   //对于仅有1byte有效数据的命令帧组装函数
    {
        byte CheckAnd = 0x00;

        //List<byte> CmdMessage = new List<byte>();
        byte[] CmdMessage = new byte[26];

        CmdMessage[0] = 0xaa;
        CheckAnd += CmdMessage[0];
        CmdMessage[1] = SlaveAddr;
        CheckAnd += CmdMessage[1];
        CmdMessage[2] = Cmd;
        CheckAnd += CmdMessage[2];
        CmdMessage[3] = Data;
        CheckAnd += CmdMessage[3];
        for (int i = 4; i < 25; i++)
        {
            CmdMessage[i] = 0x00;
            CheckAnd += CmdMessage[i];
        }
        CmdMessage[25] = CheckAnd;
        return CmdMessage;
        //return CmdMessage.ToArray();
    }

    public static byte[] CreateSetCurrentValueCmdMessage(byte SlaveAddr, byte Cmd, string Value)    //设置CC模式电流值的命令帧组装函数
    {
        byte CheckAnd = 0x00;

        Double DoubleVal = Convert.ToDouble(Value)*10000;
        UInt32 Uint32Val = (UInt32)DoubleVal;

        //List<byte> CmdMessage = new List<byte>();
        byte[] CmdMessage = new byte[26];

        CmdMessage[0] = 0xaa;
        CheckAnd += CmdMessage[0];
        CmdMessage[1] = SlaveAddr;
        CheckAnd += CmdMessage[1];
        CmdMessage[2] = Cmd;
        CheckAnd += CmdMessage[2];


        CmdMessage[3] = (byte)(Uint32Val & 0x000000ff);             CheckAnd += CmdMessage[3];
        CmdMessage[4] = (byte)((Uint32Val >> 8) & 0x000000ff);      CheckAnd += CmdMessage[4];
        CmdMessage[5] = (byte)((Uint32Val >> 16) & 0x000000ff);     CheckAnd += CmdMessage[5];
        CmdMessage[6] = (byte)((Uint32Val >> 24) & 0x000000ff);     CheckAnd += CmdMessage[6];

        for (int i = 7; i < 25; i++)
        {
            CmdMessage[i] = 0x00;
            CheckAnd += CmdMessage[i];
        }
        CmdMessage[25] = CheckAnd;
        return CmdMessage;
        //return CmdMessage.ToArray();
    }

    public static byte[] CreateSetVoltageValueCmdMessage(byte SlaveAddr, byte Cmd, string Value)    //设置CV模式电压值的命令帧组装函数
    {
        byte CheckAnd = 0x00;

        Double DoubleVal = Convert.ToDouble(Value) * 1000;
        UInt32 Uint32Val = (UInt32)DoubleVal;

        //List<byte> CmdMessage = new List<byte>();
        byte[] CmdMessage = new byte[26];

        CmdMessage[0] = 0xaa;
        CheckAnd += CmdMessage[0];
        CmdMessage[1] = SlaveAddr;
        CheckAnd += CmdMessage[1];
        CmdMessage[2] = Cmd;
        CheckAnd += CmdMessage[2];


        CmdMessage[3] = (byte)(Uint32Val & 0x000000ff);     CheckAnd += CmdMessage[3];
        CmdMessage[4] = (byte)((Uint32Val >> 8) & 0x000000ff); CheckAnd += CmdMessage[4];
        CmdMessage[5] = (byte)((Uint32Val >> 16) & 0x000000ff); CheckAnd += CmdMessage[5];
        CmdMessage[6] = (byte)((Uint32Val >> 24) & 0x000000ff); CheckAnd += CmdMessage[6];

        for (int i = 7; i < 25; i++)
        {
            CmdMessage[i] = 0x00;
            CheckAnd += CmdMessage[i];
        }
        CmdMessage[25] = CheckAnd;
        return CmdMessage;
        //return CmdMessage.ToArray();
    }

    public static byte[] CreateGetAllValueCmdMessage(byte SlaveAddr)    //读取所有值的命令帧组装函数
    {
        byte CheckAnd = 0x00;
        byte[] CmdMessage = new byte[26];

        CmdMessage[0] = 0xaa;
        CheckAnd += CmdMessage[0];
        CmdMessage[1] = SlaveAddr;
        CheckAnd += CmdMessage[1];
        CmdMessage[2] = CMD_GetAllValueAndStatusNow;
        CheckAnd += CmdMessage[2];

        for (int i = 3; i < 25; i++)
        {
            CmdMessage[i] = 0x00;
            CheckAnd += CmdMessage[i];
        }
        CmdMessage[25] = CheckAnd;
        return CmdMessage;
        //return CmdMessage.ToArray();
    }

    
    
}
