﻿/*

 * 名称：ZEL60xx_Demo
 * 
 * 程序员：王维攀
 * 
 * 创建日期：2016.03.28
 * 
 * 最后一次修改日期：2016.03.29
 * 
 * 备注：该程序是一个ZEL60xx直流电子负载的Demo程序，主要使用帧格式通讯协议发送控制命令（本Demo接收但不判断校验帧），简单实现了电子负载CC模式下的设置、回读显示。
 
 
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO.Ports;
using System.Threading;

namespace ZEL60xx_Demo
{
    public partial class Form1 : Form
    {
        //static byte SlaveAddress = 0x01; 
        SerialPort MySerial = null;
        byte[] CmdMessage_Ready = new byte[26];

        public Form1()
        {
            InitializeComponent();

            RefreshSerial();
        }

        private void RefreshSerial()    //刷新串口
        {
            for (int i = 1; i < 32; i++)
            {
                try
                {
                    MySerial = new SerialPort("COM" + i.ToString());
                    MySerial.Open();
                    MySerial.Close();
                    ComboBox_OnlineSerial.Items.Add("COM" + i.ToString());
                }
                catch (System.Exception)
                {
                    continue;
                }
            }
        }

        private void Serial_SendCmdMessage(byte[] CmdMessage)
        {
            try
            {
                int CharsLength = CmdMessage.Length;

                MySerial.Write(CmdMessage, 0, CharsLength);
                Thread.Sleep(100);
                if (SerialPort_Read() == true)
                {
                    if (Serial_Rcv[2] == Class_CmdMessgage.CMD_GetAllValueAndStatusNow)
                    {
                        Disp_OutVoltage.Text = (Convert.ToDouble ((Int32)Serial_Rcv[3] + ((Int32)Serial_Rcv[4] << 8) + ((Int32)Serial_Rcv[5] << 16) + ((Int32)Serial_Rcv[6] << 24))/1000).ToString ("f4");
                        Disp_OutCurrent.Text = (Convert.ToDouble((Int32)Serial_Rcv[7] + ((Int32)Serial_Rcv[8] << 8) + ((Int32)Serial_Rcv[9] << 16) + ((Int32)Serial_Rcv[10] << 24)) / 10000).ToString("f4");
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("错误：" + ex.Message);
            } 
        }

        public bool CheckoutAckMessage()
        {
            for (int i = 0; i < 3; i++)
            {
                if (Serial_Rcv[i] != CmdMessage_Ready[i]) return false;
            }
            return true;
        }

        static bool Serial_Open_Flag = false;
        private void Button_OpenSerial_Click(object sender, EventArgs e)
        {
            if (Serial_Open_Flag == false)
            {
                MySerial = new SerialPort(ComboBox_OnlineSerial.Text);

                MySerial.BaudRate = Convert.ToInt32(ComboBox_BaudRate.Text);
                MySerial.DataBits = 8;
                MySerial.StopBits = StopBits .One ;
                MySerial.Parity = Parity.None;

                MySerial.ReadTimeout = -1;
                MySerial.WriteBufferSize = 26;
                MySerial.ReadBufferSize = 26;

                MySerial.Open();

                Button_OpenSerial.Text = "已经打开";
                Serial_Open_Flag = true;

                CmdMessage_Ready = Class_CmdMessgage.CreateOneByteDataCmdMessage(Convert.ToByte(Input_SlaveAddr.Text), Class_CmdMessgage.CMD_SetControlMode, Class_CmdMessgage.ControlModeSel_PC);
                Serial_SendCmdMessage(CmdMessage_Ready);
            }
            else
            {
                CmdMessage_Ready = Class_CmdMessgage.CreateOneByteDataCmdMessage(Convert.ToByte(Input_SlaveAddr.Text), Class_CmdMessgage.CMD_SetControlMode, Class_CmdMessgage.ControlModeSel_Local);
                Serial_SendCmdMessage(CmdMessage_Ready); 
                MySerial = new SerialPort(ComboBox_OnlineSerial.Text);
                MySerial.Close();
                Button_OpenSerial.Text = "打开串口";
                Serial_Open_Flag = false;
            }
            
        }
    
        public byte[] Serial_Rcv = new byte[26];
        private bool SerialPort_Read()    //读串口数据
        {
            if (MySerial.IsOpen == false)
            {
                return false;
            }
            int Count = MySerial.BytesToRead;
            if (Count == 0)
            {
                return false;
            }
            Byte[] aa = new Byte[Count];
            MySerial.Read(aa, 0, Count);
            if (aa[0] != 0xaa)
            {
                return false;
            }
            Serial_Rcv = aa;
            return true;
        }

        private void Button_Get_Click(object sender, EventArgs e)
        {
            CmdMessage_Ready = Class_CmdMessgage.CreateGetAllValueCmdMessage(Convert.ToByte(Input_SlaveAddr.Text));
            Serial_SendCmdMessage(CmdMessage_Ready);
        }

        private void Button_Set_Click(object sender, EventArgs e)
        {
            CmdMessage_Ready = Class_CmdMessgage.CreateOneByteDataCmdMessage(Convert.ToByte(Input_SlaveAddr.Text), Class_CmdMessgage.CMD_SetWorkMode, Class_CmdMessgage.CMD_SetWorkMode_CC);
            Serial_SendCmdMessage(CmdMessage_Ready);
            Thread.Sleep(200);
            CmdMessage_Ready = Class_CmdMessgage.CreateSetCurrentValueCmdMessage(Convert.ToByte(Input_SlaveAddr.Text), Class_CmdMessgage.CMD_SetModeCCValue, Input_SetCurrent.Text);
            Serial_SendCmdMessage(CmdMessage_Ready);
            Thread.Sleep(200);
        }

        static bool FlagONOFF = false;
        private void Button_ONOFF_Click(object sender, EventArgs e)
        {
            if (FlagONOFF == false)
            {
                CmdMessage_Ready = Class_CmdMessgage.CreateOneByteDataCmdMessage(Convert.ToByte(Input_SlaveAddr.Text), Class_CmdMessgage.CMD_SetONOFF, Class_CmdMessgage.CMD_SetON);
                Serial_SendCmdMessage(CmdMessage_Ready);
                FlagONOFF = true;
                Button_ONOFF.Text = "OFF";
            }
            else
            {
                CmdMessage_Ready = Class_CmdMessgage.CreateOneByteDataCmdMessage(Convert.ToByte(Input_SlaveAddr.Text), Class_CmdMessgage.CMD_SetONOFF, Class_CmdMessgage.CMD_SetOFF);
                Serial_SendCmdMessage(CmdMessage_Ready);
                FlagONOFF = false;
                Button_ONOFF.Text = "ON";
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ComboBox_BaudRate.Items.Add("300");
            ComboBox_BaudRate.Items.Add("600");
            ComboBox_BaudRate.Items.Add("1200");
            ComboBox_BaudRate.Items.Add("2400");
            ComboBox_BaudRate.Items.Add("4800");
            ComboBox_BaudRate.Items.Add("9600");
            ComboBox_BaudRate.Items.Add("19200");
            ComboBox_BaudRate.Items.Add("38400");
            ComboBox_BaudRate.Items.Add("43000");
            ComboBox_BaudRate.Items.Add("56000");
            ComboBox_BaudRate.Items.Add("57600");
            ComboBox_BaudRate.Items.Add("115200");

            ComboBox_BaudRate.SelectedIndex = 7;
        }


        
    }
}
